import random


def generate_test_case(
    max_nm=20,
    max_val=10**9,
    p=None,
    special_divisibility=False,
    is_max=False,
):
    """
    Generate a test case for the divisible paths problem

    Args:
    - max_nm: Maximum value for N and M (default 20)
    - max_val: Maximum value for grid cells (default 10^9)
    - p: Specific value for P (optional)
    - special_divisibility: If True, 80% of values are divisible by P
    - is_max: If True, generate a test case with the maximum values

    Returns:
    - Tuple (N, M, P, grid) suitable for input to the solution
    """
    N = random.randint(2, max_nm)
    M = random.randint(2, max_nm)

    if is_max:
        N = max_nm
        M = max_nm

    # Choose P
    if p is None:
        # Randomly choose P
        p_choices = [
            random.randint(1, 10),  # small P
            random.randint(1, 1000),  # medium P
            random.randint(1, 100000),  # large P
            random.randint(100000, 1000000),  # very large P
        ]
        P = random.choice(p_choices)
    else:
        P = p

    # Generate grid
    grid = []
    for _ in range(N):
        row = []
        for _ in range(M):
            if special_divisibility and random.random() < 0.8:
                # Generate a value divisible by P
                row.append(P * random.randint(0, max_val // P))
            else:
                # Random value
                row.append(random.randint(0, max_val))
        grid.append(row)

    return N, M, P, grid


def write_test_case(filename, test_case):
    """Write a test case to a file"""
    N, M, P, grid = test_case

    with open(filename, "w") as f:
        f.write(f"{N} {M} {P}\n")
        for row in grid:
            f.write(" ".join(map(str, row)) + "\n")


def generate_test_suite():
    """Generate a suite of test cases"""
    test_cases = [
        generate_test_case(max_nm=5, is_max=True),
        generate_test_case(max_nm=7, is_max=True),
        generate_test_case(max_nm=10, special_divisibility=True),
        generate_test_case(max_nm=10, is_max=True),
        generate_test_case(max_nm=10, is_max=True),
        generate_test_case(max_nm=10, special_divisibility=True),
        generate_test_case(max_nm=10),
        generate_test_case(max_nm=10),
        generate_test_case(max_nm=10),
        generate_test_case(max_nm=10),
        generate_test_case(max_nm=10, is_max=True),
        generate_test_case(max_nm=10, is_max=True),
        generate_test_case(max_nm=10, special_divisibility=True, is_max=True),
        generate_test_case(max_nm=20, special_divisibility=True, is_max=True),
        generate_test_case(max_nm=10, p=2),
        generate_test_case(max_nm=10, p=2),
        generate_test_case(p=3),
        generate_test_case(p=3),
        generate_test_case(p=5, is_max=True),
        generate_test_case(p=5, is_max=True),
        generate_test_case(p=7, is_max=True),
        generate_test_case(p=7, is_max=True),
        generate_test_case(p=11),
        generate_test_case(p=100, is_max=True),
        generate_test_case(p=1000, is_max=True),
        generate_test_case(p=100000, is_max=True),
        generate_test_case(p=1000000, is_max=True),
        generate_test_case(special_divisibility=True),
        generate_test_case(p=10, special_divisibility=True, is_max=True),
        generate_test_case(p=10, special_divisibility=True, is_max=True),
        generate_test_case(p=10, special_divisibility=True, is_max=True),
        generate_test_case(p=100, special_divisibility=True, is_max=True),
        generate_test_case(p=100, special_divisibility=True, is_max=True),
        generate_test_case(max_nm=15, p=1000000, is_max=True),
        generate_test_case(max_nm=20, p=2, is_max=True),
        generate_test_case(max_nm=20, p=77, is_max=True),
    ]

    num_tests = len(test_cases)

    num_small_n_plus_m = 0
    for i, test_case in enumerate(test_cases[:num_tests], 1):
        if test_case[0] + test_case[1] <= 20:
            num_small_n_plus_m += 1

        filename = f"table.{str(i).zfill(2)}.in"
        write_test_case(filename, test_case)
        print(f"Generated {filename}")

    print(f"Generated {num_tests} test cases ({num_small_n_plus_m} with N+M <= 20)")


# Generate test suite
generate_test_suite()
print("Test suite generation complete.")
